'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = debounce;

/**
 * Original Source: https://stackoverflow.com/questions/24004791/can-someone-explain-the-debounce-function-in-javascript
 * This method will prevent functions to be called repeatedly
 * @param {Function} func Function to debounce
 * @param {Number} [delay=0] The number of milliseconds to delay the function call
 * @param {Boolean} [immediate=false]
 * Weather to call the function and then wait or vice versa
 * @returns {Function} Returns the new debounced function.
 * @example
 * // Log the event after 1000ms of the last call
 * const debounced = debounce(console.log, 1000)
 * window.addEventListener('keyup', debounced)
 *
 * // Log the event immediately and wait 1000ms for the next call
 * const debounced = debounce(console.log, 1000, true)
 * window.addEventListener('keyup', debounced)
 *
 * // If 'delay' is falsy (except for 0), the function will not be debounced
 * const debounced = debounce(console.log)
 * window.addEventListener('keyup', debounced)
 */

function debounce(func, delay, immediate) {
  var timeout = void 0;
  if (typeof func !== 'function') {
    throw new TypeError('Expected a function');
  }
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var callNow = immediate && !timeout;
    if (!delay && delay !== 0) {
      func.apply(undefined, args);
    } else {
      clearTimeout(timeout);

      timeout = setTimeout(function () {
        timeout = null;
        if (!immediate) {
          func.apply(undefined, args);
        }
      }, delay);

      if (callNow) {
        func.apply(undefined, args);
      }
    }
  };
}